@@ -48,7 +48,7 @@
 
 /* -- global variables -- */
 
-INFO info, default_info;
+INFO info;
 unsigned char deco_glob[256], deco_tune[256];
 struct SYMBOL *sym;		/* (points to the symbols of the current voice) */
 
@@ -135,7 +135,6 @@ int main(int argc,
 	info['T' - 'A'] = &notitle;
 	notitle.as.text = "T:";
 	set_format();
-	memcpy(&default_info, &info, sizeof default_info);
 	s_argc = argc;
 	s_argv = argv;
 
@@ -161,7 +160,8 @@ int main(int argc,
 				in_fname = "";		/* read from stdin */
 				continue;
 			}
-			if (p[1] != '-' && p[strlen(p) - 1] == '-')
+			if (p[1] != '-' && p[1] != 'e'
+			 && p[strlen(p) - 1] == '-')
 				c = '+'; /* switch off flags with '-x-' */
 		}
 		if (c == '+') {		/* switch off flags with '+' */
@@ -552,7 +552,6 @@ static void output_file(char *sel)
 	/* initialize if not already done */
 	if (fout == 0)
 		set_page_format();
-	memcpy(&info, &default_info, sizeof info);
 	reset_deco();
 	memcpy(&deco_tune, &deco_glob, sizeof deco_tune);
 	if (!epsf)
