@@ -1856,6 +1856,8 @@ void draw_measnb(void)
 		 || s->u == 0)
 			continue;
 		bar_num = s->u;
+		if (bar_num <= 0)
+			continue;
 		if (cfmt.measurenb == 0
 		 || (bar_num % cfmt.measurenb) != 0
 		 || s->next == 0)
@@ -2052,7 +2054,7 @@ float draw_partempo(float top,
 		shift = 1;
 		x = 0;
 /*fixme:have tempo on other voices but the 1st?*/
-		for (s = voice_tb[cursys->top_voice].sym->next;
+		for (s = voice_tb[cursys->top_voice].sym;
 		     s != 0;
 		     s = s->next) {
 			if ((g = s->extra) == 0)
@@ -2110,7 +2112,7 @@ float draw_partempo(float top,
 	h = cfmt.font_tb[PARTSFONT].size + 2 + 2;	/* + cfmt.partsspace; */
 	str_font(PARTSFONT);
 	ymin = staff_tb[0].topbar + 14;
-	for (s = voice_tb[cursys->top_voice].sym->next; s != 0; s = s->next) {
+	for (s = voice_tb[cursys->top_voice].sym; s != 0; s = s->next) {
 		if ((g = s->extra) == 0)
 			continue;
 		for (; g != 0; g = g->next)
@@ -2127,7 +2129,7 @@ float draw_partempo(float top,
 		dy = ymin + h + ht - top;
 
 	set_font(PARTSFONT);
-	for (s = voice_tb[cursys->top_voice].sym->next; s != 0; s = s->next) {
+	for (s = voice_tb[cursys->top_voice].sym; s != 0; s = s->next) {
 		if ((g = s->extra) == 0)
 			continue;
 		for (; g != 0; g = g->next)
