@@ -634,7 +634,12 @@ static void set_clef(int staff)
 			/* have a 2nd choice on beam start */
 #if 1
 #if 1
+#if 1 /* double clef pb - clef change on 2nd voice */
+			if ((s2->sflags & S_BEAM_ST)
+			 && !voice_tb[s2->voice].second)
+#else
 			if (s2->sflags & S_BEAM_ST)
+#endif
 				s3 = s2;
 #else
 			if ((s3->sflags & S_BEAM_ST) == 0)
@@ -1553,7 +1558,7 @@ static float set_space(struct SYMBOL *s)
 /* -- set the width and space of all symbols -- */
 /* this function is called once for the whole tune
  * then, once per music line up to the first sequence */
-static void set_allsymwidth(void)
+static void set_allsymwidth(struct SYMBOL *last_s)
 {
 	struct VOICE_S *p_voice;
 	struct SYMBOL *s, *s2, *s3;
@@ -1634,7 +1639,7 @@ static void set_allsymwidth(void)
 			new_val = set_space(s2);
 			if (space < new_val)
 				space = new_val;
-			if ((s2 = s2->ts_next) == 0)
+			if ((s2 = s2->ts_next) == last_s)
 				break;
 		} while (!(s2->sflags & S_SEQST));
 		/* set the spaces at start of sequence */
@@ -1645,7 +1650,7 @@ static void set_allsymwidth(void)
 			s->shrink = shrink;
 			s->space = space;
 		}
-		if ((s = s2) == 0)
+		if ((s = s2) == last_s)
 			break;
 	}
 
@@ -1662,13 +1667,13 @@ static void set_allsymwidth(void)
 	if (space == 0)
 		return;
 	shrink = 0;
-	for (s = tsfirst; s != 0; s = s->ts_next) {
+	for (s = tsfirst; s != last_s; s = s->ts_next) {
 		if (s->shrink != 0)
 			shrink += s->shrink;
 		if (s->as.type == ABC_T_NOTE)
 			break;
 	}
-	if (s != 0 && shrink < space) {
+	if (s != last_s && shrink < space) {
 		while (!(s->sflags & S_SEQST))
 			s = s->ts_prev;
 		s->shrink += space - shrink;
@@ -2230,7 +2235,7 @@ static void set_yval(struct SYMBOL *s)
 /* -- set the pitch of the notes according to the clefs -- */
 /* also set the vertical offset of the symbols */
 /* this function is called only once per tune */
-static void set_pitch(void)
+static void set_pitch(struct SYMBOL *last_s)
 {
 	struct SYSTEM *sy;
 	struct SYMBOL *s;
@@ -2248,7 +2253,7 @@ static void set_pitch(void)
 		delta = delta_tb[sy->staff[staff].clef.type];
 		staff_clef[staff] = delta + sy->staff[staff].clef.line * 2;
 	}
-	for (s = tsfirst; s != 0; s = s->ts_next) {
+	for (s = tsfirst; s != last_s; s = s->ts_next) {
 		struct SYMBOL *g;
 		int np, m, pav;
 
@@ -2295,6 +2300,10 @@ static void set_pitch(void)
 			}
 			set_yval(s);
 			break;
+		case KEYSIG:
+			s->pits[0] = staff_clef[staff];
+			set_yval(s);
+			break;
 		default:
 			set_yval(s);
 			break;
@@ -2845,11 +2854,8 @@ static void init_music_line(void)
 				if (s->sflags & S_SEQST)
 					break;
 		}
-		if (s != 0)
-			s->ts_prev->ts_next = 0;
-		set_allsymwidth();
-		if (s != 0)
-			s->ts_prev->ts_next = s;
+		set_pitch(last_s);
+		set_allsymwidth(s);
 	}
 }
 
@@ -2934,8 +2940,9 @@ static void set_global(void)
 				int delta;
 				struct SYMBOL *g;
 
-				i = p_voice - voice_tb;
-				i = sy->voice[i].clef.type;
+//				i = p_voice - voice_tb;
+//				i = sy->voice[i].clef.type;
+				i = p_voice->clef.type;
 				if (!p_voice->forced_clef
 				    || i == PERC)
 					continue;
@@ -3059,7 +3066,7 @@ static void set_global(void)
 	}
 	init_music_line();
 	insert_meter &= ~1;		/* keep the 'first line' flag */
-	set_pitch();			/* adjust the note pitches */
+	set_pitch(0);			/* adjust the note pitches */
 }
 
 /* -- return the left indentation of the staves -- */
@@ -3804,8 +3811,14 @@ static void set_piece(void)
 	int staff;
 
 	/* reset the staves */
-	for (staff = 0; staff <= nstaff; staff++)
-		staff_tb[staff].y = 0;		/* staff system not computed */
+	sy = cursys;
+	for (staff = 0; staff <= nstaff; staff++) {
+		p_staff = &staff_tb[staff];
+		p_staff->y = 0;		/* staff system not computed */
+		p_staff->clef.stafflines = sy->staff[staff].clef.stafflines;
+		if (sy->staff[staff].clef.staffscale != 0)
+			p_staff->clef.staffscale = sy->staff[staff].clef.staffscale;
+	}
 
 	/* search the next end of line,
 	 * set the repeat measures, (remove some dble bars?) */
@@ -3820,6 +3833,17 @@ static void set_piece(void)
 				}
 			}
 		}
+		if (s->type == STAVES) {
+			sy = sy->next;
+			for (staff = 0; staff <= sy->nstaff; staff++) {
+				p_staff = &staff_tb[staff];
+//--fixme: what is this comparison ?
+				if (sy->staff[staff].clef.stafflines > p_staff->clef.stafflines)
+					p_staff->clef.stafflines = sy->staff[staff].clef.stafflines;
+				if (sy->staff[staff].clef.staffscale != 0)
+					p_staff->clef.staffscale = sy->staff[staff].clef.staffscale;
+			}
+		}
 #if 0
 		if (s->type == BAR
 		    && s->next != 0
@@ -3851,19 +3875,6 @@ static void set_piece(void)
 #endif
 	}
 
-	/* set the global staff system */
-	for (staff = 0; staff <= nstaff; staff++)
-		staff_tb[staff].clef.stafflines = 0;
-	for (sy = cursys; sy != 0; sy = sy->next) {
-		for (staff = 0; staff <= sy->nstaff; staff++) {
-			p_staff = &staff_tb[staff];
-			if (sy->staff[staff].clef.stafflines > p_staff->clef.stafflines)
-				p_staff->clef.stafflines = sy->staff[staff].clef.stafflines;
-			if (sy->staff[staff].clef.staffscale != 0)
-				p_staff->clef.staffscale = sy->staff[staff].clef.staffscale;
-		}
-	}
-
 	for (staff = 0; staff <= nstaff; staff++) {
 		p_staff = &staff_tb[staff];
 		p_staff->botbar = p_staff->clef.stafflines <= 3 ? 6 : 0;
@@ -4141,6 +4152,7 @@ static void cut_symbols(void)
 static void gen_init(void)
 {
 	struct SYMBOL *s;
+//	int staff;
 
 	for (s = tsfirst; s != 0; s = s->ts_next) {
 		if (s->extra != 0)
@@ -4151,19 +4163,34 @@ static void gen_init(void)
 		case TIMESIG:
 			continue;
 		case FMTCHG:
+			if (s->extra != 0)
+				output_ps(s, 127);
 			delsym(s);
 			if (tsfirst == 0)
 				return;
 			continue;
 		case STAVES:
+#if 0
+			for (staff = 0; staff < MAXSTAFF; staff++) {
+				cursys->next->staff[staff].clef.type =
+					cursys->staff[staff].clef.type;
+				cursys->next->staff[staff].clef.line =
+					cursys->staff[staff].clef.line;
+				cursys->next->staff[staff].clef.octave =
+					cursys->staff[staff].clef.octave;
+				cursys->next->staff[staff].clef.invis =
+					cursys->staff[staff].clef.invis;
+			}
+#endif
 			cursys = cursys->next;
 			delsym(s);
 			if (tsfirst == 0)
 				return;
 			break;
 		}
-		break;
+		return;
 	}
+	tsfirst = 0;			/* no note */
 }
 
 /* -- update the clefs at start of line -- */
@@ -4221,7 +4248,7 @@ void output_music(void)
 	set_stems();			/* set the stem lengths */
 	if (first_voice->next != 0)	/* when multi-voices */
 		set_overlap();		/* shift the notes on voice overlap */
-	set_allsymwidth();		/* set the symbols width */
+	set_allsymwidth(0);		/* set the symbols width */
 
 	lwidth = ((cfmt.landscape ? cfmt.pageheight : cfmt.pagewidth)
 		- cfmt.leftmargin - cfmt.rightmargin)
