@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2009 Jean-Franois Moine
+ * Copyright (C) 1998-2011 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -43,6 +43,8 @@ static struct SYSTEM *parsys;		/* current system while parsing */
 struct FORMAT dfmt;		/* current global format */
 unsigned short *micro_tb;	/* ptr to the microtone table of the tune */
 
+static INFO info_glob;		/* global info definitions */
+
 static int lyric_nb;			/* current number of lyric lines */
 static struct SYMBOL *lyric_start;	/* 1st note of the line for w: */
 static struct SYMBOL *lyric_cont;	/* current symbol when w: continuation */
@@ -433,10 +435,11 @@ static void voice_dup(void)
 		for ( ; s != 0; s = s->next) {
 			s2 = (struct SYMBOL *) getarena(sizeof *s2);
 			memcpy(s2, s, sizeof *s2);
-			if (p_voice2->sym != 0) {
+			s2->prev = p_voice2->last_sym;
+			s2->next = 0;
+			if (p_voice2->sym != 0)
 				p_voice2->last_sym->next = s2;
-				s2->prev = p_voice2->last_sym;
-			} else	p_voice2->sym = s2;
+			else	p_voice2->sym = s2;
 			p_voice2->last_sym = s2;
 			s2->voice = voice;
 			s2->staff = p_voice2->staff;
@@ -447,9 +450,9 @@ static void voice_dup(void)
 				s2->sflags |= S_FLOATING;
 			else	s2->sflags &= ~S_FLOATING;
 			s2->ly = 0;
-			if (s2->type != GRACE)
-				continue;
 			g = s2->extra;
+			if (g == 0)
+				continue;
 			g2 = (struct SYMBOL *) getarena(sizeof *g2);
 			memcpy(g2, g, sizeof *g2);
 			s2->extra = g2;
@@ -547,6 +550,15 @@ static void staves_init(void)
 		case CLEF:
 			if (s->as.u.clef.type < 0)
 				break;
+			{
+				int scale, lines;
+
+				scale = sy->voice[s->voice].clef.staffscale;
+				lines = sy->voice[s->voice].clef.stafflines;
+				sy->voice[s->voice].clef = s->as.u.clef;
+				sy->voice[s->voice].clef.staffscale = scale;
+				sy->voice[s->voice].clef.stafflines = lines;
+			}
 			continue;	/* normal clef change */
 		case KEYSIG:
 		case TIMESIG:
@@ -559,12 +571,21 @@ static void staves_init(void)
 			continue;
 		}
 
-		/* #lines or scale of staff found */
+		/* CLE with change of #lines or scale of staff */
 		voice = s->voice;
 		if (staves == 0) {
 			staves = s;	/* create a new staff system */
 			new_sy = (struct SYSTEM *) getarena(sizeof *new_sy);
 			memcpy(new_sy, sy, sizeof *new_sy);
+			for (voice = 0; voice < MAXVOICE; voice++) {
+				if (new_sy->voice[voice].range < 0
+				    || new_sy->voice[voice].second)
+					continue;
+				staff = new_sy->voice[voice].staff;
+				memcpy(&new_sy->staff[staff].clef,
+					&new_sy->voice[voice].clef,
+					sizeof (struct clef_s));
+			}
 			new_sy->next = sy->next;
 			sy->next = new_sy;
 			sy = new_sy;
@@ -1020,6 +1041,34 @@ static void get_staves(struct SYMBOL *s)
 		if (p_voice->sym != 0)
 			flags = 1;
 	}
+	if (flags == 0			/* if first %%staves */
+	    || (maxtime == 0 && staves_found < 0)) {
+		for (voice = 0; voice < MAXVOICE; voice++)
+			parsys->voice[voice].range = -1;
+	} else {
+
+		/* create a new staff system and
+		 * link the staves in a voice which is seen from
+		 * the previous system - see sort_all */
+		p_voice = curvoice;
+		if (p_voice->ignore) {
+			for (voice = 0; voice < MAXVOICE; voice++) {
+				if (parsys->voice[voice].range >= 0) {
+					curvoice = &voice_tb[voice];
+					break;
+				}
+			}
+/*fixme: should check if voice < MAXVOICE*/
+		}
+		sym_link(s, STAVES);	/* link the staves in the current voice */
+		s->as.state = ABC_S_HEAD; /* (output PS sequences immediately) */
+		parsys->nstaff = nstaff;
+		system_new();
+	}
+	curvoice = first_voice;
+	staves_found = maxtime;
+
+	/* initialize the voices */
 	for (voice = 0, p_voice = voice_tb;
 	     voice < MAXVOICE;
 	     voice++, p_voice++) {
@@ -1028,19 +1077,6 @@ static void get_staves(struct SYMBOL *s)
 		p_voice->ignore = 0;
 		p_voice->time = maxtime;
 	}
-	if (flags == 0			/* if first %%staves */
-	    || (maxtime == 0 && staves_found < 0)) {
-		for (voice = 0; voice < MAXVOICE; voice++)
-			parsys->voice[voice].range = -1;
-	} else {			/* else, have a new staff system */
-		parsys->nstaff = nstaff;
-		sym_link(s, STAVES);
-		s->as.state = ABC_S_HEAD; /* (output PS sequences immediately) */
-		system_new();
-	}
-	staves_found = maxtime;
-
-	/* initialize the voices */
 	dup_voice = MAXVOICE;
 	range = 0;
 	p_staff = s->as.u.staves;
@@ -1059,6 +1095,9 @@ static void get_staves(struct SYMBOL *s)
 			p_voice2 = &voice_tb[voice];
 			memcpy(p_voice2, p_voice, sizeof *p_voice2);
 			p_voice2->next = 0;
+			p_voice2->sym = p_voice2->last_sym = 0;
+			p_voice2->tblts[0] = p_voice2->tblts[1] = 0;
+			p_voice2->clone = -1;
 			while (p_voice->clone > 0)
 				p_voice = &voice_tb[p_voice->clone];
 			p_voice->clone = voice;
@@ -1176,12 +1215,10 @@ static void get_info(struct SYMBOL *s,
 		     int info_type,
 		     struct abctune *t)
 {
-	INFO *inf;
 	char *p;
 	int old_lvl;
 
 	/* change global or local */
-	inf = s->as.state == ABC_S_GLOBAL ? &default_info : &info;
 	old_lvl = lvlarena(s->as.state != ABC_S_GLOBAL);
 
 	switch (info_type) {
@@ -1228,7 +1265,7 @@ static void get_info(struct SYMBOL *s,
 		switch (s->as.state) {
 		case ABC_S_GLOBAL:
 		case ABC_S_HEAD:
-			(*inf)['P' - 'A'] = s;
+			info['P' - 'A'] = s;
 			break;
 		case ABC_S_TUNE: {
 			struct VOICE_S *p_voice;
@@ -1256,7 +1293,7 @@ static void get_info(struct SYMBOL *s,
 		switch (s->as.state) {
 		case ABC_S_GLOBAL:
 		case ABC_S_HEAD:
-			(*inf)['Q' - 'A'] = s;
+			info['Q' - 'A'] = s;
 			break;
 		default:
 			if (cfmt.printtempo)
@@ -1314,8 +1351,8 @@ static void get_info(struct SYMBOL *s,
 newtune:
 		if (!epsf)
 			write_buffer();	/* flush stuff left from %% lines */
-		dfmt = cfmt;		/* save the format at start of tune */
-		memcpy(&info, &default_info, sizeof info);
+		dfmt = cfmt;		/* save format and info */
+		memcpy(&info_glob, &info, sizeof info_glob);
 		info['X' - 'A'] = s;
 		if (info_type == 'T')
 			info['T' - 'A'] = s;
@@ -1327,11 +1364,11 @@ addinfo:
 		if (info_type >= 'A' && info_type <= 'Z') {
 			struct SYMBOL *prev;
 
-			prev = (*inf)[info_type - 'A'];
+			prev = info[info_type - 'A'];
 			if (prev == 0
 			    || (prev->as.state == ABC_S_GLOBAL
 				&& s->as.state != ABC_S_GLOBAL)) {
-				(*inf)[info_type - 'A'] = s;
+				info[info_type - 'A'] = s;
 				break;
 			}
 			while (prev->next != 0)
@@ -1743,8 +1780,8 @@ void do_tune(struct abctune *t,
 	} else	write_buffer();
 
 	if (info['X' - 'A'] != 0) {
-		cfmt = dfmt;	/* restore the format at start of tune */
-		info['X' - 'A'] = 0;
+		cfmt = dfmt;	/* restore format and info */
+		memcpy(&info, &info_glob, sizeof info);
 	}
 	clrarena(1);
 	
@@ -2437,7 +2474,8 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 		if (strcmp(w, "newpage") == 0) {
 			if (epsf)
 				return as;
-			gen_ly(0);
+			generate();
+			buffer_eob();
 			write_buffer();
 			use_buffer = 0;
 			if (isdigit((unsigned char) *p))
@@ -2603,7 +2641,8 @@ irepeat:
 				voice_init();
 			}
 #endif
-			get_staves(s);
+			if (*p != '\0')
+				get_staves(s);
 			return as;
 		}
 		if (strcmp(w, "sysstaffsep") == 0) {
